import pandas as pd

from kywy.client.kawa_decorators import outputs, inputs


@inputs(measure1=float, measure2=float)
@outputs(measure3=float)
def execute(df: pd.DataFrame):
    print('run: simple_join_script')
    print('[1]: Data from KAWA')
    print(df)
    df['measure3'] = df['measure1'] + df['measure2']
    raise Exception('Test error hohoho')



from ...kawa_base_e2e_test import KawaBaseTest
import pandas as pd
import uuid
import tempfile
from ....client.kawa_client import KawaClient as K


class TestComputationWithNumericFilters(KawaBaseTest):

    @classmethod
    def setUpClass(cls):
        unique_id = 'resource_{}'.format(uuid.uuid4())
        print(tempfile.gettempdir())
        KawaBaseTest.setUpClass()
        cls.df = cls._df_for_tests()
        cls.sheet_name = unique_id
        loader = cls.kawa.new_data_loader(df=cls.df.copy(deep=True), datasource_name=unique_id)
        loader.create_datasource()
        loader.load_data(reset_before_insert=True, create_sheet=True)

    def test_without_filters(self):
        # when
        computed_df = (self.kawa
                       .sheet(sheet_name=self.sheet_name)
                       .select('integer')
                       .compute())

        # then
        self.assertListEqual(list(computed_df['integer']), [1, 2, 3, 4, 5])

    def test_gt_filter(self):
        self._apply_and_check_filter(K.col('integer').gt(4), [5])
        self._apply_and_check_filter(K.col('integer').gt(4.1), [5])
        self._apply_and_check_filter(K.col('integer').gt(1), [2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').gt(-11), [1, 2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').gt(11), [])

    def test_gte_filter(self):
        self._apply_and_check_filter(K.col('integer').gte(4), [4, 5])
        self._apply_and_check_filter(K.col('integer').gte(4.1), [5])
        self._apply_and_check_filter(K.col('integer').gte(2), [2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').gte(-2), [1, 2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').gte(29), [])

    def test_lt_filter(self):
        self._apply_and_check_filter(K.col('integer').lt(4), [1, 2, 3])
        self._apply_and_check_filter(K.col('integer').lt(3.9), [1, 2, 3])
        self._apply_and_check_filter(K.col('integer').lt(2), [1])
        self._apply_and_check_filter(K.col('integer').lt(-11), [])
        self._apply_and_check_filter(K.col('integer').lt(11), [1, 2, 3, 4, 5])

    def test_lte_filter(self):
        self._apply_and_check_filter(K.col('integer').lte(4), [1, 2, 3, 4])
        self._apply_and_check_filter(K.col('integer').lte(3.9), [1, 2, 3])
        self._apply_and_check_filter(K.col('integer').lte(2), [1, 2])
        self._apply_and_check_filter(K.col('integer').lte(-11), [])
        self._apply_and_check_filter(K.col('integer').lte(11), [1, 2, 3, 4, 5])

    def test_eq_filter(self):
        self._apply_and_check_filter(K.col('integer').eq(4), [4])
        self._apply_and_check_filter(K.col('integer').eq(3.9), [])
        self._apply_and_check_filter(K.col('integer').eq(2), [2])
        self._apply_and_check_filter(K.col('integer').eq(-11), [])
        self._apply_and_check_filter(K.col('integer').eq(11), [])

    def test_ne_filter(self):
        self._apply_and_check_filter(K.col('integer').ne(4), [1, 2, 3, 5])
        self._apply_and_check_filter(K.col('integer').ne(3.9), [1, 2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').ne(2), [1, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').ne(-11), [1, 2, 3, 4, 5])
        self._apply_and_check_filter(K.col('integer').ne(11), [1, 2, 3, 4, 5])

    def _apply_and_check_filter(self, f, expected):
        df = (self.kawa
              .sheet(sheet_name=self.sheet_name)
              .filter(f)
              .select('integer')
              .compute())
        self.assertListEqual(list(df['integer']), expected)

    @staticmethod
    def _df_for_tests():
        data = {
            'text': ['c1', 'c1', 'c1', 'c2', 'c2'],
            'integer': [1, 2, 3, 4, 5],
        }
        return pd.DataFrame(data)



import pandas as pd

from kywy.client.kawa_decorators import inputs, outputs, secrets


@inputs(measure1=float)
@outputs(measure3=float)
@secrets(delta='delta_in_letters')
def tata(df: pd.DataFrame, delta):
    print(df)
    df['measure3'] = df['measure1'] + len(delta)
    print(df)
    return df



import pandas as pd
from kywy.client.kawa_decorators import inputs, outputs


@inputs(measure1=float)
@outputs(measure3=float)
def execute(df: pd.DataFrame):
    print('run: simple_join_script')
    print('[1]: Data from KAWA')
    print(df)
    df['measure3'] = df['measure1'] * 5
    print('[2]: Data after script operation')
    print(df)
    return df


@inputs(measure1=float)
@outputs(measure3=float)
def execute2(df: pd.DataFrame):
    print('run: simple_join_script')
    print('[1]: Data from KAWA')
    print(df)
    df['measure3'] = df['measure1'] * 5
    print('[2]: Data after script operation')
    print(df)
    return df



import pandas as pd

from kywy.client.kawa_decorators import inputs, outputs


@inputs(pk=int, measure1=float, measure2=float)
@outputs(measure3=float)
def execute(df: pd.DataFrame):
    df['measure3'] = df['measure1'] + df['measure2']
    return df



import datetime
import logging
from datetime import date

import pandas as pd

from kywy.client.kawa_decorators import outputs, inputs, secrets


@inputs(decimal=float, integer=int, date=date, datetime=datetime.datetime, boolean=bool, text=str)
@outputs(decimal_res=float, integer_res=int, date_res=date, datetime_res=datetime.datetime, boolean_res=bool,
         text_res=str)
@secrets(secret_param_name='secret_name_in_kawa')
def execute(df: pd.DataFrame, secret_param_name: str):
    script_logger = logging.getLogger('script-logger')
    script_logger.info(df)
    df['decimal_res'] = df.apply(lambda row: decimal_func(row['decimal']), axis=1)
    df['integer_res'] = df.apply(lambda row: integer_func(row['integer']), axis=1)
    df['date_res'] = df.apply(lambda row: date_func(row['date']), axis=1)
    df['datetime_res'] = df.apply(lambda row: datetime_func(row['datetime']), axis=1)
    df['boolean_res'] = df.apply(lambda row: bool_function(row['boolean']), axis=1)
    df['text_res'] = df.apply(lambda row: str_function(row['text'], secret_param_name), axis=1)
    return df


def integer_func(i: int):
    return i + 1


def decimal_func(f: float):
    return f + 1.1


def date_func(d: date):
    return d + datetime.timedelta(days=1)


def datetime_func(dt: datetime.datetime):
    return dt + datetime.timedelta(hours=1)


def bool_function(b: bool):
    return not b


def str_function(s: str, secret: str):
    return s.capitalize() + secret
